from django.apps import AppConfig
from django.conf import settings

class LangChainSearchConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "langchain_search"

    def ready(self):
        # Set default settings if not already set
        if not hasattr(settings, 'LANGCHAIN_SEARCH_QUERY_PARAM'):
            setattr(settings, 'LANGCHAIN_SEARCH_QUERY_PARAM', 'q')
        if not hasattr(settings, 'LANGCHAIN_SEARCH_MODEL_SETTINGS'):
            setattr(settings, 'LANGCHAIN_SEARCH_MODEL_SETTINGS', {})




from django.conf import settings
from langchain_search.util.langchain import get_llm_filter


class LangChainSearchMixin():
    def get_queryset(self):
        queryset = super().get_queryset()
        search_query = self.request.GET.get(
            settings.LANGCHAIN_SEARCH_QUERY_PARAM,
            'q'
        )
        if search_query:
            filter = get_llm_filter(
                queryset,
                search_query,
                **settings.LANGCHAIN_SEARCH_MODEL_SETTINGS,
            )
            queryset = queryset.filter(**filter)
        return queryset



import logging
from typing import TypedDict
from django.db.models.query import QuerySet

from langchain.chains import LLMChain
from langchain.prompts import PromptTemplate
from langchain_core.output_parsers import JsonOutputParser
from langchain_openai import ChatOpenAI

from langchain_search.util.django import (
    create_filter_model_from_django,
)

logger = logging.getLogger(__name__)


class ModelSettings(TypedDict):
    """
    Settings for the model, including the model name and any additional parameters.
    """

    model: str = "gpt-4.1"
    temperature: float = 0.0
    max_tokens: int = 1000
    timeout: int = 60
    max_retries: int = 2


def get_llm_filter(queryset: QuerySet, query_text: str, **kwargs: ModelSettings) -> dict:
    """
    Generate a filter using an LLM based on the provided query text.
    EXAMPLE:

    ```
        query_text = "at least $80000 medium size company remote in Matthewshaven"

        {
            'salary__gte': 80000,
            'remote': True,
            'location__ilike': 'Matthewshaven',
            'company_employee_count__gte': 51,
            'company_employee_count__lte': 250
        }
    ```

    ```
    query_text = "salary of at least $100000 in San Francisco"

    {
        'salary__gte': 100000,
        'location__ilike': 'San Francisco'
    }
    ```
    """
    if not queryset or not query_text:
        return {}

    opts = {
        "model": kwargs.get("model", "gpt-4.1"),
        "temperature": kwargs.get("temperature", 0.0),
        "max_tokens": kwargs.get("max_tokens", 1000),
        "timeout": kwargs.get("timeout", 60),
        "max_retries": kwargs.get("max_retries", 2),
    }

    llm = ChatOpenAI(**opts)#.with_structured_output(
        #method="json_mode",
    #) # Not sure why this doesn't work

    template = """
    You are a filter generator.
    Given a query, generate a JSON object that can be used to filter a table in a database.
    Each value should only be used on a single field based on what is most suitable
    where the values belong to one of the field examples, the field should take priority.
    {format_instructions}
    {query}
    """
    filter_model = create_filter_model_from_django(queryset.model)

    parser = JsonOutputParser(pydantic_object=filter_model)
    prompt = PromptTemplate(
        template=template,
        input_variables=["query"],
        partial_variables={"format_instructions": parser.get_format_instructions()},
    )

    logger.info("LLM prompt: %s", prompt)

    chain = LLMChain(llm=llm, prompt=prompt)

    logger.info("query  text: %s", query_text)
    response = chain.run({"query": query_text})

    logger.info("LLM response: %s", response)

    response = parser.parse(response)

    smart_filter = {k: v for k, v in response.items() if v not in (None, 0, "", [], {})}

    return smart_filter



import decimal

from django.db.models import (
    BooleanField,
    CharField,
    DecimalField,
    FloatField,
    IntegerField,
    TextField,
    DateTimeField,
)
from pydantic import create_model


def create_filter_model_from_django(model_class):
    field_definitions = {}
    for field in model_class._meta.get_fields():
        if field.is_relation or not hasattr(field, "get_internal_type"):
            continue

        name = field.name

        if isinstance(field, (CharField, TextField)):
            name = f"{name}__icontains"
            field_definitions[name] = (str | None, None)
        elif isinstance(field, BooleanField):
            field_definitions[name] = (bool | None, None)
        elif isinstance(field, (IntegerField)):
            field_definitions[f"{name}__gte"] = (int | None, None)
            field_definitions[f"{name}__lte"] = (int | None, None)
        elif isinstance(field, (FloatField)):
            field_definitions[f"{name}__gte"] = (float | None, None)
            field_definitions[f"{name}__lte"] = (float | None, None)
        elif isinstance(field, (DecimalField)):
            field_definitions[f"{name}__gte"] = (decimal.Decimal | None, None)
            field_definitions[f"{name}__lte"] = (decimal.Decimal | None, None)
        elif isinstance(field, DateTimeField):
            field_definitions[f"{name}__gte"] = (str | None, None)
            field_definitions[f"{name}__lte"] = (str | None, None)

    return create_model(f"{model_class.__name__}Filter", **field_definitions)
